
#include "dac.h"
#include "lcd.h"

/*
DAC1220 settings

	On MMB3, the DAC1220 is connected using SDIO and SCLK only - /CS is tied to ground.
	The DAC1220 on this board is WRITE-ONLY.
	
	SCLK is on P3.0
	SDIO is on P3.1

	DAC output goes from 0 to 5V, but goes through differential buffer to ADC input on MMB3
	Differential output is ±5V, a single-ended output is available
	
	DAC Vout	OUTPUT+		OUTPUT-		Differential Output		Single-Ended Output 
	0.0V		5.0V		0.0V			+5V						+5.0V
	2.5V		2.5V		2.5V			 0V						 2.5V
	5.0V		0.0V		5.0V			-5V						 0.0V

	We will write to the part MSB first, both byte- and bit-wise.
	
	data format - DF - two's complement vout = 2VREF(code+2^19)/(2^20) - default
	adaptive filtering enabled - default
	output disconnected during cal - default
	
	Note that the OCR and FSR registers are full 24-bit numbers, but the DIR register
	is a 16- or 20-bit number, which must be left-justified, i.e. the MS bit MUST be 
	the MS bit in the MS byte... this is important if you write to these as if they are
	24- or 32-bit registers!
	
*/

#define SCLK_LO	P3OUT&=~0x01
#define SCLK_HI P3OUT|=0x01
#define SDIO_LO P3OUT&=~0x02
#define SDIO_HI P3OUT|=0x02
#define BLOCK_SZ 32
#define TABLE_SIZE 32

//int sinetable[TABLE_SIZE], costable[TABLE_SIZE];

u8 dac_res=0; /* DAC resolution; global, for convenience*/


void dac_reset (void)
{
	u16 i;
	
	lcd_showstr("RESET  ");
	dint();  /* Turn off the interrupts*/
	
	SCLK_HI;
	for(i=0;i<T3;i++);

	SCLK_LO;
	for(i=0;i<T2;i++);
	
	SCLK_HI;
	for(i=0;i<T4;i++);
	
	SCLK_LO;
	for(i=0;i<T2;i++);
	
	SCLK_HI;
	/*for(i=0;i<T3;i++);
	
	SCLK_LO;
	for(i=0;i<T2;i++);
	*/
	SCLK_HI;
	for(i=0;i<T5;i++);
	
	SCLK_LO;
	
	eint();  /* Turn interrupts back on*/
	
}

void dac_set_resolution (u8 res)
{
	/* 0 = 16 bit, 1 = 20bit */
	dac_res = res;
	dac_write(DAC_WRITE | DAC_ONE_BYTE | DAC_CMR_BYTE0, (s32)(res*(0x80)));
	
}

void dac_self_calibrate (void)
{
	u32 i;
	lcd_showstr("SELFCAL");
	dac_set_op_mode(dac_res, (s32)(MODE_SELFCAL));
	for(i=0;i<TSC;i++);
	
}

void dac_set_op_mode (u8 res, u8 mode)
{
	u16 cmrreg;
	
	cmrreg = (res*0x8000) | mode;
	
	dac_write(DAC_WRITE | DAC_TWO_BYTES | DAC_CMR_REG, (s32)cmrreg);
}

void dac_init (void)
{
	u32 i;
	dac_reset();
	for(i=0;i<TSC;i++);
	dac_set_resolution(BIT_20);
	dac_self_calibrate();
	dac_set_op_mode(dac_res, MODE_NORMAL);

	
}


/*
	dac_write
	
	This function writes to any register in the DAC1220. It takes a 32-bit number
	as input - note that for DAC output, the number must be left-justified! The dac_output
	function handles formatting this for you.
*/
void dac_write (u8 insr, s32 regval)
{	
	u8 i;
	s8 j;
	u8 t;
	s8 numbytes;
	u8 regbyte[3];
	
	if ((insr&0x07)>15) return;
	
	numbytes = ((insr&0x60)>>5);

	regbyte[2] = (u8)((regval&0x00FF0000)>>16);		//MSB
	regbyte[1] = (u8)((regval&0x0000FF00)>>8);
	regbyte[0] = (u8)(regval&0x000000FF);	    //LSB
	
	//send command byte with INSR
	for (i=0;i<8;i++)
	{
		SCLK_HI;
		if (insr&0x80){
			SDIO_HI;
		}else{
			SDIO_LO;
		}
		for(t=0;t<100;t++);
		SCLK_LO;
		for(t=0;t<100;t++);
		insr = insr<<1;
	}
	
	for(i=0;i<255;i++);
	
	for(j=numbytes;j>=0;j--)
	{
		
		//write 1 byte at a time
		for (i=0;i<8;i++)
		{
			SCLK_HI;
			if (regbyte[j]&0x80){
				SDIO_HI;
			}else{
				SDIO_LO;
			}
			for(t=0;t<100;t++);
			SCLK_LO;
			for(t=0;t<100;t++);
			regbyte[j] = regbyte[j]<<1;
		}
		
		for(i=0;i<250;i++);
	
	}
	
}



/*
	dac_output
	This function takes a normal 32-bit number, right-justified, and
	formats it properly for the DAC1220 and sends the value to the DAC.
*/
void dac_output(s32 dacval)
{
	s32 regval;
	
	regval = (dacval&0x000FFFFF)<<4;
	dac_write(DAC_WRITE | DAC_THREE_BYTES | DAC_DIR_REG, regval);
}

/*void InitTables(int period)
{
		int y;
		for (y=0; y<TABLE_SIZE; y++)
		{sinetable[y] = (unsigned int)((sin(2*3.141593659*period*y /
			TABLE_SIZE) + 1.0) * 32767);
		costable[y] = (unsigned int)((cos(2*3.141593659*period*y /
			TABLE_SIZE) + 1.0) * 32767);
		}
		}
		*/
